<?php if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Plugin Name: Featured Slider
 * Plugin URI:
 * Description: It's like a featured image, but more, it's a slider!
 * Version: 1.3
 * Author: Haris Ainur Rozak
 * Author URI: https://harisrozak.github.io
 */

define("FESLIDER", 'feslider');
define("FESLIDER_DIR", plugin_dir_path( __FILE__ ));
define("FESLIDER_URL", plugin_dir_url( __FILE__ ));

require_once(FESLIDER_DIR . 'inc/default-options.php');

Class FESLIDER {
	function __construct() {
		$this->options = feslider_options();

		add_action('add_meta_boxes', array($this, 'add_meta_boxes'));
		add_action('admin_enqueue_scripts', array($this, 'admin_enqueue_scripts'), 10, 1);
		add_action('save_post', array($this, 'save_post'));
		add_shortcode('feslider', array($this, 'add_shortcode'));
		add_action('admin_menu', array($this, 'admin_menu'));
		add_action('admin_init', array($this, 'admin_init'));
		add_filter('the_content', array($this, 'the_content'));
		add_filter('get_post_metadata', array($this, 'get_post_metadata'), 10, 4);
		add_filter('post_thumbnail_html', array($this, 'post_thumbnail_html'), 10, 5);
	}

	function add_meta_boxes() {
		$post_types = $this->options['post_types'];

		add_meta_box(
			'feslider_meta_box',
			'Feslider - Featured Slider',
			array($this, 'meta_box_callback'),
			$post_types,
			'normal',
			'high'
	    );
	}

	function meta_box_callback($post) {
		// Use nonce for verification
    	wp_nonce_field(plugin_basename( __FILE__ ), 'feslider_noncename');

    	$feslider = get_post_meta($post->ID, 'feslider', true);
    	$feslider_images = isset($feslider['images']) ? $feslider['images'] : array();

		require_once(FESLIDER_DIR . 'inc/feslider-uploader.php');
		require_once(FESLIDER_DIR . 'inc/meta-box.php');
	}

	function admin_enqueue_scripts($hook) {
		// show only on posts new form
    	if($hook != 'post-new.php' && $hook != 'post.php' && $hook != 'toplevel_page_feslider-settings') return;

    	// style
    	wp_enqueue_style("wp-jquery-ui-dialog");
		wp_enqueue_style('feslider-admin', FESLIDER_URL . "lib/style-admin.css");

		// script
		wp_enqueue_script('jquery');
		wp_enqueue_script('jquery-ui-sortable');
		wp_enqueue_script('jquery-ui-dialog');
	}

	function save_post($post_id) {
		// verify if this is an auto save routine
	    // If it is our form has not been submitted, so we dont want to do anything
	    if(defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) return;

	    // verify this came from the our screen and with proper authorization
	    // because save_post can be triggered at other times
	    $_POST['feslider_noncename'] = isset($_POST['feslider_noncename']) ? $_POST['feslider_noncename'] : '';
	    if(!wp_verify_nonce( $_POST['feslider_noncename'], plugin_basename( __FILE__ ))) return;

	    // Check permissions
	    if(isset($_POST['post_type'])) {
	        if('page' == $_POST['post_type']) {
	            if(! current_user_can('edit_page', $post_id)) return;
	        }
	        else {
	            if(! current_user_can('edit_post', $post_id)) return;
	        }
	    }

		$feslider = array(
			'images'   => array(),
			'count'    => 0,
		);

	    // Getting the images.
	    if ( isset( $_POST['feslider_image_id'] ) && is_array( $_POST['feslider_image_id'] ) ) {
			$images_id = array_map("absint", $_POST['feslider_image_id']);
			$images_url = array_map("esc_url", $_POST['feslider_image_url']);
			$images_thumb = array_map("esc_url", $_POST['feslider_image_thumb']);
			$feslider['count'] = count($images_id);

			if(is_array($images_id) && $feslider['count'] > 0) {
				foreach ($images_id as $key => $value) {
					$feslider['images'][$key] = array(
						'id' => $value,
						'url' => $images_url[$key],
						'thumb' => $images_thumb[$key]
					);
				}
			}
		}

	    update_post_meta($post_id, 'feslider', $feslider);
	}

	function add_shortcode($attr) {
		global $post;
		$post_types = $this->options['post_types'];

		// show only on selected post type(s)
		if(! in_array($post->post_type, $post_types)) return '';

		// replace featured image or not
		if($this->is_replaced($post->ID)) return '';

		return $this->display_shortcode();
	}

	function display_shortcode() {
		global $post;

		// jquery
		wp_enqueue_script('jquery');

		// slider pro
		wp_enqueue_style('feslider', FESLIDER_URL . "lib/slider-pro/css/slider-pro.min.css");
		wp_enqueue_style('feslider-frontend', FESLIDER_URL . "lib/style-frontend.css");
		wp_enqueue_script('feslider', FESLIDER_URL . "lib/slider-pro/js/jquery.sliderPro.min.js", array('jquery'), "1.6.0", true);

		// load some arguments
		$post_id = isset($attr['post_id']) ? $attr['post_id'] : $post->ID;
		$feslider = get_post_meta($post_id, 'feslider', true);
		$options = feslider_options();

		// load feslider script
		wp_add_inline_script('feslider', "
			jQuery('#feslider-{$post_id}').sliderPro({
				width: {$options['width']},
				height: {$options['height']},
				responsive: {$options['responsive']},
				imageScaleMode: '{$options['imageScaleMode']}',
				centerImage: {$options['centerImage']},
				allowScaleUp: {$options['allowScaleUp']},
				arrows: {$options['arrows']},
				fadeArrows: {$options['fadeArrows']},
				buttons: {$options['buttons']},
				slideDistance: {$options['slideDistance']},
				fullScreen: {$options['fullScreen']},
				autoplay: {$options['autoplay']},
				slideAnimationDuration: {$options['slideAnimationDuration']},
				fade: {$options['fade']},
				shuffle: {$options['shuffle']},
				orientation: '{$options['orientation']}',
				loop: {$options['loop']},
				waitForLayers: true,
				fadeFullScreen: true,
			});
		");

		ob_start();
		require(FESLIDER_DIR . 'inc/shortcode.php');

		$output = ob_get_contents();
		ob_end_clean();

		return $output;
	}

	function admin_init() {
		register_setting('feslider', 'feslider');
	}

	function admin_menu() {
		add_menu_page(
			'Feslider',
			'Feslider',
			'moderate_comments',
			'feslider-settings',
			array($this, 'admin_settings_page'),
			'dashicons-carrot',
			100
		);
	}

	function admin_settings_page() {
		wp_enqueue_style('select2', FESLIDER_URL . "lib/select2/select2.min.css");
		wp_enqueue_script('select2', FESLIDER_URL . "lib/select2/select2.min.js");

		require_once(FESLIDER_DIR . 'inc/settings-page.php');
	}

	function the_content($content) {
		$post_ID = $GLOBALS['post']->ID;
		$slider = "[feslider]";

		// has shorcode or not
		if(has_shortcode($content, 'feslider') || $this->is_replaced($post_ID)) {
			return $content;
		}
		else {
			return $slider . $content;
		}
	}

	function is_replaced($post_ID) {
		// replace featured image is working only [enter]
		// if the featured image use get_the_post_thumbnail or the_post_thumbnail
		$feslider = get_post_meta($post_ID, 'feslider', true);
		$is_feslider = $feslider && $feslider['count'] > 0;

		return $is_feslider && $this->options['replace_featured_image'] == 'true';
	}

	function get_post_metadata($null, $object_id, $meta_key, $single) {
		// filter on request featured image only
		if($meta_key == '_thumbnail_id') {
			if($this->is_replaced($object_id)) {
				return false;
			}
			else {
				return $null;
			}
		}
	}

	function post_thumbnail_html($html, $post_ID, $thumnnail_ID, $size, $attr) {
		if($this->is_replaced($post_ID)) {
			return $this->display_shortcode();
		}
		else {
			return $html;
		}
	}
}

$GLOBALS['feslider'] = new FESLIDER();
